/**************************************************************************

	en25t80.c

	Serial Flash Memory EN25T80, SPI mode control module

	Copyright (C) 2010 S.Suwa <www.suwa-koubou.jp>


***************************************************************************/

/** INCLUDES **************************************************************/
#include <p30f3012.h>
#include "pic_232.h"
#include "pic_print.h"
#include "pic_delay.h"


/** DEFINITION ************************************************************/
// Flash Memory read, write, erase etc, command byte
#define	cmd_WREN	0x06	// write enable
#define	cmd_WRDI	0x04	// write disable, exit OTP mode
#define	cmd_RDSR	0x05	// read status register
#define	cmd_WRSR	0x01	// write status register
#define	cmd_READ	0x03	// read data bytes
#define	cmd_FREAD 	0x0B	// fast read data bytes
#define	cmd_PP		0x02	// page programing
#define	cmd_SE		0x20	// sector erase
#define	cmd_BE		0xD8	// block erase
#define	cmd_CE		0xC7	// chip erase
#define	cmd_DP		0xB9	// to deep power-down
#define cmd_RLDP	0xAB	// release deep power-down, and read Device ID
#define	cmd_RDMID	0x90	// read Manufacture ID and Device ID
#define	cmd_RDID	0x9F	// read identification
#define	cmd_OTP		0x3A	// to OTP mode

// status register bits define
#define	sts_WIP		0x01		// write in progress
#define	sts_WEL		0x02		// write enable latch
#define sts_BP		0x1C		// Block Protect
#define	sts_MODE	0x60		// SPI mode=00, extend SP2 mode=01
#define sts_SRP		0x80		// status register protect


// Flash Memory Chip Select, active low
#define	CS_ON()		LATCbits.LATC15=0
#define	CS_OFF()	LATCbits.LATC15=1


/** FUNCTION PROTOTYPE ****************************************************/
void	read_id(unsigned char *id_buf);
void	read_mid(unsigned char *id_buf);
unsigned char	read_status(void);
void	write_status(unsigned char data);
void	read_data(unsigned long addr, unsigned short read_bytes, unsigned char *read_buf);
void	write_enable(void);
void	write_data(unsigned long addr, unsigned short write_bytes, unsigned char *write_buf);
void	sector_erase(unsigned long addr);
void	block_erase(unsigned long addr);
void	chip_erase(void);
void	erase_sub(unsigned char cmd, unsigned long addr);
void	deep_power_down(void);
void	release_deep(void);
void	read_otp_data(unsigned char *read_buf);

unsigned long get_addr(void);
unsigned short	get_data(unsigned char *buf, int count);
void	display_data(unsigned long addr);

void	SpiSend(unsigned char send_byte);
unsigned char	SpiSendRec(unsigned char send_byte);
unsigned char	SpiRec(void);

/** VARIABLE **************************************************************/
unsigned char membuf[256];
char strbuf[32];


/*=========================================================================
	EN25T80 access test program
==========================================================================*/

void	en25t80_test(void)
{
	unsigned long addr;
	unsigned char status;
	unsigned char protect;
	unsigned short size;
	int	c, i;


	rs_putstr("\r\n*** EN25T80 Flash Memory Test Program **************\r\n");

	release_deep();
	protect = 0;
	for(;;){
		rs_putstr("\r\nPlease command input.\r\n");
		rs_putstr("  (R)ead,(W)rite,(S)tatus,(P)rotect,(I)d,(M)id,(O)TP,\r\n");
		rs_putstr("  (D)eep power down, re(L)ease deep power down,\r\n");
		rs_putstr("  (1)sector erase,(2) block erase, (3)chip erase.\r\n");
		rs_putstr("  when you hit Ctrl+'C' key, return to system.\r\n");

		while((c = rs_getwchr()) == -1);
		
		switch(c){
		case 0x03:	// hit Ctrl+C
			goto exit;

		case '1':
			sector_erase(get_addr());
			rs_putstr("complete\r\n");
			break;

		case '2':
			block_erase(get_addr());
			rs_putstr("complete\r\n");
			break;

		case '3':
			chip_erase();
			rs_putstr("complete\r\n");
			break;

		case 'R':
		case 'r':
			for(i=0;i<256;i++) membuf[i] = 0;
			read_data((addr = get_addr()), 256, membuf);
			display_data(addr);
			break;

		case 'W':
		case 'w':
			for(i=0;i<256;i++) membuf[i] = 0;
			addr = get_addr();
			size = get_data(membuf, 256);
			write_data(addr, size, membuf);
			rs_putstr("complete\r\n");
			break;

		case 'S':
		case 's':
			status = read_status();
			myprintf("\r\nStatus=%02x\r\n", status);
			break;

		case 'P':
		case 'p':
			write_status(protect);	// protect set/reset
			protect ^= 0x1c;
			rs_putstr("Ok!\r\n");
			break;

		case 'O':
		case 'o':
			for(i=0;i<256;i++) membuf[i] = 0;
			read_otp_data(membuf);
			display_data(0L);
			break;
			
		case 'I':
		case 'i':
			read_id(membuf);
			myprintf("\r\nManufactureID=%02x\r\n", membuf[0]);
			myprintf("MemoryType ID=%02x\r\n", membuf[1]);
			myprintf("Capacity ID=%02x\r\n", membuf[2]);
			break;

		case 'M':
		case 'm':
			read_mid(membuf);
			myprintf("\r\nManufactureID=%02x\r\n", membuf[0]);
			myprintf("Device ID=%02x\r\n", membuf[1]);
			break;


		case 'D':
		case 'd':
			deep_power_down();
			rs_putstr(" to deep power down mode.\r\n");
			break;

		case 'L':
		case 'l':
			release_deep();
			rs_putstr(" release deep power down mode.\r\n");
			break;
		}
	}
exit:
	rs_putstr("\r\n... return to system.\r\n");
}

/*-------------------------------------------------------------------------
	get addr
--------------------------------------------------------------------------*/

unsigned long get_addr(void)
{
	unsigned long addr;
	int	c;

	rs_set_echo(ECHO_ON);
	rs_putstr("addr=>");

	addr = 0L;
	for(;;){
		while((c = rs_getwchr()) == -1);
		if(c == '\r') break;
		addr <<= 4;
		addr |= htoi((char *)&c);
	}
	rs_set_echo(ECHO_OFF);
	return addr;
}


/*-------------------------------------------------------------------------
	get data
--------------------------------------------------------------------------*/

unsigned short	get_data(unsigned char *buf, int count)
{
	int	size;
	int	c;

	rs_set_echo(ECHO_ON);
	rs_putstr("data=>");

	for(size = 0; size < count; size++){
		while((c = rs_getwchr()) == -1);
		if(c == '\r') break;
		*buf++ = c;
	}
	rs_set_echo(ECHO_OFF);
	return size;

}


/*-------------------------------------------------------------------------
	display memory dump data
       +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F
0F0000 00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f
0F0010 10 11 12 13 14 15 16 17 18 19 1a 1b 1c 1d 1e 1f
--------------------------------------------------------------------------*/

void	display_data(unsigned long addr)
{
	int	i, j;
	
	rs_putstr("\r\n       +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F\r\n");
	for(i = 0; i < 16; i++){
		int_to_2hex(strbuf, (addr >> 16) & 0xff);
		int_to_4hex(strbuf+2, addr & 0xffff);
		rs_putstr(strbuf);
		for(j = 0; j < 16; j++){
			int_to_2hex(strbuf+1, membuf[i*16+j]);
			strbuf[0] = ' ';
			rs_putstr(strbuf);
		}
		rs_putstr("\r\n");
		addr += 16;
	}
}


/*-------------------------------------------------------------------------
	read identification, 
		1st: manufacture ID
		2nd: memory type ID
		3rd: capacity ID
--------------------------------------------------------------------------*/

void	read_id(unsigned char *id_buf)
{
	CS_ON();
	SpiSend(cmd_RDID);
	*id_buf++ = SpiRec();	// Manufacture ID
	*id_buf++ = SpiRec();	// Device ID High
	*id_buf++ = SpiRec();	// Device ID Low
	CS_OFF();
}


/*-------------------------------------------------------------------------
	read Manufacture/DeviceID, 
		1st: manufacture ID
		2nd: device ID
--------------------------------------------------------------------------*/

void	read_mid(unsigned char *id_buf)
{
	CS_ON();
	SpiSend(cmd_RDMID);
	SpiSend(0);
	SpiSend(0);
	SpiSend(0);
	*id_buf++ = SpiRec();	// Manufacture ID
	*id_buf++ = SpiRec();	// Device ID High
	CS_OFF();
}


/*-------------------------------------------------------------------------
	read status register
--------------------------------------------------------------------------*/

unsigned char	read_status(void)
{
	unsigned char	stat;

	// select memory
	CS_ON();

	// command send
	SpiSend(cmd_RDSR);

	// rec status code
	stat = SpiRec();

	// deselect memory
	CS_OFF();

	return stat;
}


/*-------------------------------------------------------------------------
	write status register
--------------------------------------------------------------------------*/

void	write_status(unsigned char data)
{
	// write enable
	write_enable();

	// select memory
	CS_ON();

	// command send
	SpiSend(cmd_WRSR);
	SpiSend(data);

	// deselect memory
	CS_OFF();

}

/*-------------------------------------------------------------------------
	read data
--------------------------------------------------------------------------*/

void	read_data(unsigned long addr, unsigned short read_bytes, unsigned char *read_buf)
{
	// select memory
	CS_ON();

	// command send
	SpiSend(cmd_READ);

	// 24bit addr send
	SpiSend((addr >> 16) & 0xff);
	SpiSend((addr >> 8) & 0xff);
	SpiSend(addr & 0xff);

	while(read_bytes){
		// receive data
		*read_buf++ = SpiRec();
		read_bytes--;
	}

	// deselect memory
	CS_OFF();
}

/*-------------------------------------------------------------------------
	otp read data
--------------------------------------------------------------------------*/

void	read_otp_data(unsigned char *read_buf)
{
	// entered OTP mode
	CS_ON();
	SpiSend(cmd_OTP);
	CS_OFF();

	// read OTP sector
	read_data(0xFF000L, 256, read_buf);

	// exit OTP mode
	CS_ON();
	SpiSend(cmd_WRDI);
	CS_OFF();

}


/*-------------------------------------------------------------------------
	write enable
--------------------------------------------------------------------------*/

void	write_enable(void)
{
	unsigned char status;

	// check prev write or erase in progress
	do {
		status = read_status();
	}while(status & sts_WIP);

	CS_ON();
	SpiSend(cmd_WREN);
	CS_OFF();

	// check write enable latch
	do {
		status = read_status();
	}while(!(status & sts_WEL));

}



/*--------------------------------------------------------------------------
	write data
--------------------------------------------------------------------------*/

void	write_data(unsigned long addr, unsigned short write_bytes, unsigned char *write_buf)
{
	unsigned char status;

	// write enable
	write_enable();

	//--- data write --------------
	CS_ON();
	// page program command
	SpiSend(cmd_PP);
	// 24bit addr
	SpiSend((addr >> 16) & 0xff);
	SpiSend((addr >> 8) & 0xff);
	SpiSend(addr & 0xff);
	// write data
	while(write_bytes){
		SpiSend(*write_buf++);
		write_bytes--;
	}
	// complete
	CS_OFF();

	// wait until write complete
	do {
		status = read_status();
	}while(status & sts_WIP);

}


/*--------------------------------------------------------------------------
	sector erase
--------------------------------------------------------------------------*/

void	sector_erase(unsigned long addr)
{
	erase_sub(cmd_SE, addr);
}


/*--------------------------------------------------------------------------
	block erase
--------------------------------------------------------------------------*/

void	block_erase(unsigned long addr)
{
	erase_sub(cmd_BE, addr);
}


/*--------------------------------------------------------------------------
	chip erase
--------------------------------------------------------------------------*/

void	chip_erase(void)
{
	erase_sub(cmd_CE, 0L);
}


void	erase_sub(unsigned char cmd, unsigned long addr)
{
	unsigned char status;


	// write enable
	write_enable();

	// erase 
	CS_ON();
	SpiSend(cmd);
	if(cmd != cmd_CE){
		// 24bit addr
		SpiSend((addr >> 16) & 0xff);
		SpiSend((addr >> 8) & 0xff);
		SpiSend(addr & 0xff);
	}
	CS_OFF();

	// wait until write complete
	do {
		status = read_status();
	}while(status & sts_WIP);


}


/*--------------------------------------------------------------------------
	to deep power down
--------------------------------------------------------------------------*/

void	deep_power_down(void)
{
	// select memory
	CS_ON();

	// command send
	SpiSend(cmd_DP);

	// deselect memory
	CS_OFF();

}


/*--------------------------------------------------------------------------
	release deep power down
--------------------------------------------------------------------------*/

void	release_deep(void)
{
	// select memory
	CS_ON();

	// command send
	SpiSend(cmd_RLDP);

	// deselect memory
	CS_OFF();

}

/*--------------------------------------------------------------------------
	SpiEnable()	SPI enabled
--------------------------------------------------------------------------*/

void	SpiEnable(void)
{
	CS_OFF();
	SPI1STATbits.SPIEN = 1;
}


/*--------------------------------------------------------------------------
	SpiDisable() SPI disable
--------------------------------------------------------------------------*/

void	SpiDisable(void)
{
	CS_OFF();
	SPI1STATbits.SPIEN = 0;
}

/*--------------------------------------------------------------------------
	SpiSend()	received data ignore
--------------------------------------------------------------------------*/

void	SpiSend(unsigned char send_byte)
{
	unsigned char dummy;

	// wait until TXBUF empty
	while(SPI1STATbits.SPITBF);

	// write to TXBUF
	SPI1BUF = send_byte;

	// wait until data send out
	//  (wait for data received complete)
	while(!SPI1STATbits.SPIRBF);

	dummy = SPI1BUF;	// clear receive buffer 
}


/*--------------------------------------------------------------------------
	SpiSendRec()	data send and data receive 
--------------------------------------------------------------------------*/

unsigned char	SpiSendRec(unsigned char send_byte)
{
	// wait until TXBUF empty
	while(SPI1STATbits.SPITBF);

	// write to TXBUF
	SPI1BUF = send_byte;

	// wait until data send out
	//  (wait for data received complete)
	while(!SPI1STATbits.SPIRBF);

	return SPI1BUF;
}


/*--------------------------------------------------------------------------
	SpiRec()	data receive( dummy send ) 
--------------------------------------------------------------------------*/

unsigned char	SpiRec(void)
{
	// wait until TXBUF empty
	while(SPI1STATbits.SPITBF);

	// write to dummy data
	SPI1BUF = 0xff;

	// wait until data send out
	//  (wait for data received complete)
	while(!SPI1STATbits.SPIRBF);

	return SPI1BUF;
}

/*** end of en25t80.c ******************************************************/
