/*********************************************************************************************

	templogger.c

	Copyright (C) 2010-2012 S.Suwa <www.suwa-koubou.jp>

	temperature measurement logging for dsPIC30F3012

	CPU:                dsPIC30F3012(use +3.3V)
	Flash Memory:       EON EN25T80 (8Mbits,1MBYTE)
	Real Time Clock:    EPSONTOYOCOM RTC-8564NB
	Temperature sencer: LM61BIZ (-30 to 100) 	

	CPU Clock(+3.3V)
	 internal FRC, PLLx8, Fcy=64MHz(7.37Mx8), Tcy=16MHz(14.74M)

	CPU I/O Pin ASSIGN
	 SPI for EN25T80
	  SPI Data Out: RB4  EN25T80 DI
	  SPI Data In:  RB5  EN25T80 DO
	  SPI Clock:    RB6  EN25T80 CLK
	  SPI CS:       RC15 EN25T80 CS#

	 I2C for RTC-8564NB
	  I2C Clock:    RB4  RTC8564 SCL
	  I2C Data IO:  RB5  RTC8564 SDA
	  INT1:         RD0  RTC8564 INT#
	  INT2:         RB7  push button for wakeup

	 UART to PC
	  U1ATX:        RC13
	  U1ARX:        RC14

	 ADC for LM61BIZ
	  AN3           RB3 LM61BIZ 


	                 MCR#(1) (18)AVDD ---------- +3V3
	                  RB0(2) (17)AVSS ---------- GND
	                  RB1(3) (16)RB6/SCK1 ------ EN25T80-CLK
	                  RB2(4) (15)RB7 ----------- Push Button  
	LM61BIZ ----- AN3/RB3(5) (14)VDD ----------- +3V3
	                 OSC1(6) (13)VSS ----------- GND
	EN25T80-CS# ---- RC15(7) (12)RB5/SDI1/SDA -- EN25T80-DO/RTC8564-SDA
	UART TX -- U1ATX/RC13(8) (11)RB4/SDO1/SCL -- EN25T80-DI/RTC8564-SCL
	UART RX -- U1ARX/RC14(9) (10)INT1/RD0 ------ RTC8564-INT#


*********************************************************************************************/

#include <p30f3012.h>
#include "typedef.h"
#include "pic_232.h"
#include "pic_print.h"
#include "pic_delay.h"
#include "en25t80.h"
#include "rtc8564.h"


// Device Configuration
//_FOSC(CSW_FSCM_OFF & FRC_PLL16) // no clock change, internal RC osc 7.37M x PLL16
_FOSC(CSW_FSCM_OFF & FRC_PLL8) // no clock change, internal RC osc 7.37M x PLL8
_FWDT(WDT_OFF) // watch dog timer off
_FBORPOR(PBOR_OFF & MCLR_EN)
_FGS(CODE_PROT_OFF)
_FICD(ICS_PGD)


void	initial_io(void);
void	display_temp(void);
short	temp_measure(void);
void	temp_logging(void);
void	logger(void);
void	set_memaddr(void);
void	log_read(void);

static	unsigned long memaddr;		// logdata stored flash memory address
static	unsigned char logdata[32];	// ASCII YYMMDDHH 8Byte
									// temp  DDDD  signed short 2byte x 12(5minutes interval)

/*--------------------------------------------------------------------------
	INT2 interrupt
	switch for wakeup interrupt
--------------------------------------------------------------------------*/
static volatile unsigned char switch_interrupt;

void __attribute__ ((__interrupt__, no_auto_psv)) _INT2Interrupt(void)
{
	switch_interrupt = 1;	// set aplication interrupt flag
	IFS1bits.INT2IF = 0;	// clear interrupt flag
}


/*=====================================================================================================
	main
=====================================================================================================*/

int	main(void)
{
	int c;
//	char	pbuf[6];

	// system initialize
	initial_io();
	rs_initialize();


	for(;;){
		rs_putstr("\r\n");
		rs_putstr("**** Temperature auto logger ************************************\r\n");
		rs_putstr(" F: flash memory mentenance.\r\n");
		rs_putstr(" T: display now temperature.\r\n");
		rs_putstr(" C: real time clock mentenance.\r\n");
		rs_putstr(" L: logging temperature.\r\n");
		rs_putstr(" R: log read.\r\n");
		rs_putstr("Input Command=>");
 
		while((c = rs_getwchr()) == -1);

		if(c > ' ' && c < 0x7f){
			rs_putwchr(c);
		}else{
			//rs_putstr(int_to_2hex(pbuf, c));
			myprintf("%02x", c);
		}
		rs_putstr("\r\n");

		// exec command
		switch(c){
		/***
		case 'Q':		// ready enquiry ?
		case 'q':
			rs_putchr('R');	// Ready responce!
			break;
		***/
		case 'F':
		case 'f':
			SpiEnable();
			en25t80_test();
			SpiDisable();
			break;

		case 'T':
		case 't':
			display_temp();
			break;

		case 'C':
		case 'c':
			rtc8564_test();
			break;


		case 'L':
		case 'l':
			temp_logging();
			break;

		case 'R':
		case 'r':
			rs_putstr("\r\n");
			log_read();
			break;

		default:
			break;
		}
	}

	return 0;

}


/*-----------------------------------------------------------------------------------------------------
	initial I/O, interrupt
-----------------------------------------------------------------------------------------------------*/

void	initial_io(void)
{
	//--- setting interrupt ---------------------------------------
	INTCON1 = 0;			// NSTDIS=0
	INTCON2bits.INT1EP = 1;	// interrupt detect falling edge. 
	INTCON2bits.INT2EP = 1;	
	IEC1bits.INT1IE = 1;
	IEC1bits.INT2IE = 1;

	//--- setting I/O port ----------------------------------------
	ADPCFG = 0xfff7;	// AN3 enable, other Analog input disable
	TRISB = 0xffff;		// PORTB is all input
	TRISC = 0xffff;		// PORTC is all input.
	TRISD = 0xffff;		// PORTD is all input.

	//--- setting ADC ---------------------------------------------
	ADCON1 = 0x00E6;
		// ADON=0:	A/D Operation Mode
		//	   =0   A/D conerter is off
		// ADSIDL=0: Stop in idle Mode
		//	   =0   Continue module operation in idle mode
		// FORM=00:	Data Output Format
		//	   =00  12bits integer(DOUT=0000 dddd dddd dddd)
		// SSRC=111: Conversion Trigger Source select
		//	   =111  internal counter ends sampling and starts conversion(auto convert)
		// ASAM=1:	A/D Sampling Auto-Start bit
		//     =1   Sampling begins immediately after conversion completes. SAMP bit autoset
		//     =0   Sampling begins when SAMP bit set
		// SAMP=1:	A/D Sample Enable bit
		//     =1   At least one A/D sample/hold amplifier is sampling
		// DONE=0:	A/D Conversion Status bit
		//     =1	A/D Conversion is done
	 	//     =0   A/D Conversion isn't done
	ADCON2 = 0x0404;
		// VCFG=0:	reference voltage is AVdd, AVss
		// CSCNA=1:	scaned input channel 
		// BUFS=x:	Buffer Fill Status bit
		// SMPI=0:	interrupt at one sampling (AN3 only) 
		// BUFM=0:	stored 16bits word.
		// ALTS=0:	used only MUX A.
	ADCON3 = 0x1F3F;
		// SAMC=31:	auto sampling time 31xTad
		// ADRC=0:	use system clock
		// ADCS=0x3F(63):  Tad is 32 x Tcy
	ADCHS = 0x0000;		// egnore channel set, auto scan.(by ADCON2bits.CSCNA=1)
	ADCSSL = 0x0008;	// scan channel AN3
	//ADCON1bits.ADON = 1; // turn ADC ON

	//--- setting SPI ---------------------------------------------
	LATCbits.LATC15 = 1;	// RC15 CS#, to HIGH Level
	TRISCbits.TRISC15 = 0;	// RC15 set output.

	// for EN25T80 Flash Memory
	// 	SCLOCK normal low or High
	// 	DOUT out fall edg, EN25T80 read rising edge
	// 	DIN  rising IN, EN25T80 out fall edge
	// 	Normal Low CKP = 0, CKE=1 / Normal High CKP = 1, CKE = 0
	//	SDI data sampling, rising edge, SMP = 0
	SPI1STAT = 0;	// SPI1 is disable
	SPI1CON = 0b0000000001111001;
		// DISSCK=0,DISSDO=0,MODE16=0
		// SMP=0,CKE=0,SSEN=0,CKP=1,master mode
		// second prescale 2:1(110),
		// first prescale 16:1(01),  14740000/32=460625Hz
	// SPI1STATbits.SPIEN = 1;	// start SPI1

	//--- setting I2C ---------------------------------------------
	I2CCON = 0;		// disable I2C, set parameter default value.
	I2CBRG = 93;	// Fsc=400KHZ, Fcy=7.37x8=58.96MHz,
					// (58.96MHZ/400KHZ-58.96MHZ/1.111111MHZ)-1=147.4-53.06-1=94.34-1=93
	I2CADD = 0x57;	// slave module own address, not use. 0x57 is same 0xAE.


}


/*=====================================================================================================
	display temperature
=====================================================================================================*/

void	display_temp(void)
{
//	char pbuf[20];
	short temp;

	temp = temp_measure();

	rs_putstr("\r\n\r\n**** Now Temp. ************************************\r\n");
	//rs_putstr("ADC Out Value="); rs_putstr(int_to_4hex(pbuf, temp)); rs_putstr("\r\n");
	myprintf("ADC Out Value=%04x\r\n", temp);
	temp =(short)(((long)temp * 3300L + 2048L)/4096L - 600L);
	//rs_putstr("Temperature="); rs_putstr(int_to_decimal(pbuf, temp)); rs_putstr("\r\n");
	myprintf("Temperature=%d\r\n", temp);
	
}



/*-----------------------------------------------------------------------------------------------------
	temperature measurement
-----------------------------------------------------------------------------------------------------*/

short	temp_measure(void)
{
	short temp;

	ADCON1bits.ADON = 1;
	ADCON1bits.SAMP = 1;
	while(!ADCON1bits.DONE);
	ADCON1bits.DONE = 0;
	temp = ADCBUF0;
	ADCON1bits.ADON = 0;
	return temp;
}


/*=====================================================================================================
	temp. measurement and logging
=====================================================================================================*/

void	temp_logging(void)
{
	int	c;
	unsigned char tm[7];

	rs_putstr("Check the Real Time Value, Interval interrupt timer, \r\n");
	rs_putstr("and Flash Memory erase, ... Are you ready?\r\n");
	rs_putstr("hit enter to Go, hit ctrl+C to cancel.\r\n");

	while((c = rs_getwchr()) != 0x03 && c != '\r');
	if(c == 0x03){
		rs_putstr(" ... cancel.\r\n");
		return;
	}

	// read Now time and store time value into logbuffer.
	rtcReadTime(tm);
	logdata[0] = ((tm[tm_YEAR] & 0xf0) >> 4) +'0';
	logdata[1] =  (tm[tm_YEAR] & 0x0f) +'0';
	logdata[2] = ((tm[tm_MONTH] & 0xf0) >> 4) +'0';
	logdata[3] =  (tm[tm_MONTH] & 0x0f) +'0';
	logdata[4] = ((tm[tm_DAY] & 0xf0) >> 4) +'0';
	logdata[5] =  (tm[tm_DAY] & 0x0f) +'0';
	logdata[6] = ((tm[tm_HOUR] & 0xf0) >> 4) +'0';
	logdata[7] =  (tm[tm_HOUR] & 0x0f) +'0';

	// set flash address
	set_memaddr();


	rs_putstr("\r\n ... to Go auto measuring and logging.\r\n\r\n");
	delay_ms(1);

	for(;;){
		switch_interrupt = 0;
		asm("PWRSAV	#0");	// to sleep
		if(switch_interrupt) break;	// when switch was pushed, logging stop. 
		logger();
	}
	rs_putstr("... stop auto logging.\r\n");

}


/*-----------------------------------------------------------------------------------------------------
	logger
	this function was called by every minutes interval.
-----------------------------------------------------------------------------------------------------*/

void	logger(void)
{
	unsigned char tm[7];
	unsigned char minutes;
	short	temp, idx;


	// read Now time.
	rtcReadTime(tm);

	// Now time isn't x0min or x5min.
	minutes = tm[tm_MIN] & 0x0f;
	if(minutes != 0 && minutes != 5) return;

	// measure temperature
	temp = temp_measure();
	temp += temp_measure();
	temp += temp_measure();
	temp += temp_measure();
	temp >>= 2;	// average 4th times measure 	

	// convert to temp C value from A/D value.
	temp =(short)(((long)temp * 3300L + 2048L)/4096L - 600L);

	// log save
	minutes = ((tm[tm_MIN] & 0xf0)>>4) * 10 + (tm[tm_MIN] & 0x0f);
	idx = (minutes / 5) * 2 + 8;
	logdata[idx] = (temp & 0xff00) >> 8;
	logdata[idx+1] = (temp & 0x00ff);

	// Now time 00min, store time data to logdata top.
	if(minutes == 0){
		logdata[0] = ((tm[tm_YEAR] & 0xf0) >> 4) +'0';
		logdata[1] =  (tm[tm_YEAR] & 0x0f) +'0';
		logdata[2] = ((tm[tm_MONTH] & 0xf0) >> 4) +'0';
		logdata[3] =  (tm[tm_MONTH] & 0x0f) +'0';
		logdata[4] = ((tm[tm_DAY] & 0xf0) >> 4) +'0';
		logdata[5] =  (tm[tm_DAY] & 0x0f) +'0';
		logdata[6] = ((tm[tm_HOUR] & 0xf0) >> 4) +'0';
		logdata[7] =  (tm[tm_HOUR] & 0x0f) +'0';
	}

	// Now time 55min, save logdata to Flash Memory.
	if(minutes == 55){
		SpiEnable();
		release_deep();
		write_status(0);	// do unprotect
		write_data(memaddr, 32, logdata);
		write_status(0x1c);	// do protect
		deep_power_down();
		SpiDisable();

		memaddr += 32;
		for(idx = 0; idx < 32; idx++) logdata[idx] = 0xff;
	}

}

/*-----------------------------------------------------------------------------------------------------
	search & set memaddr
-----------------------------------------------------------------------------------------------------*/

void	set_memaddr(void)
{
	unsigned char buf[32];

	SpiEnable();
	release_deep();

	for(memaddr = 0L;;memaddr += 32){
		read_data(memaddr, 32, buf);
		if(buf[0] == 0xff) break;
	}

	deep_power_down();
	SpiDisable();
}


/*-----------------------------------------------------------------------------------------------------
	read and display logging data 
-----------------------------------------------------------------------------------------------------*/

void	log_read(void)
{
	unsigned char buf[32];
	char	pbuf[10];
	int		i, val;

	SpiEnable();
	release_deep();

	for(memaddr = 0L;;memaddr += 32){
		read_data(memaddr, 32, buf);
		if(buf[0] == 0xff) break;

		rs_putstr("20");
		for(i = 0; i < 8; i++){
			rs_putwchr(buf[i]);
		}
		for(i = 0; i < 12; i++){
			rs_putwchr(',');
			val = buf[i *2 + 8] << 8;
			val += buf[i *2 + 1+8];
			int_to_ndecimal(pbuf, 3, val);
			pbuf[4] = 0;
			pbuf[3] = pbuf[2];
			pbuf[2] = '.';
			rs_putstr(pbuf);
		}
		rs_putstr("\r\n");
	}

	deep_power_down();
	SpiDisable();
	
}


/*** end of templogger.c ********************************************************************************/
